from main import Downloader

if __name__ == "__main__":
    # TODO dont know how package this python code into a
    # service run from terminal this will do for now
    downloader = Downloader()
    downloader.fefmalsmc()
